#include "radio_functions.h"


static int fd;
static int multipler;
static int  mix_fd;

struct mix_channel
{
  char *label;              

  int left;                  
  int right;
  int dev_num;

} mix_channel[20];

//init mixe device
int init_mixer(char *device)
{
	mix_fd = open(device, O_RDWR);

	if  (mix_fd < 0)
		return -1;
	else
	return 1;
}

//close mixer device
void close_mixer(void)
{
	if (mix_fd > 0)
		close(mix_fd);
}



void set_volume(int vol, int dev)
{
  unsigned char temp[4];
  int channel=1;
  int left = vol;
  int right = vol;		
  mix_channel[channel].left = left;
  mix_channel[channel].right = right;
  mix_channel[channel].dev_num = dev;
  temp[0]=(unsigned char)left;
  temp[1]=(unsigned char)right;
  temp[2]=temp[3]=0;
  ioctl(mix_fd, MIXER_WRITE(mix_channel[channel].dev_num), temp);
}

int get_volume(int dev)
{
  unsigned char temp[4];
  unsigned char left; 
  unsigned char right;
  int channel=1;
  mix_channel[channel].dev_num = dev;
  ioctl(mix_fd, MIXER_READ(mix_channel[channel].dev_num), &temp);
  left = temp[0];
  right = temp[1];
	if ( temp[0] >= temp[1] )
  {
      if (left <= 100)
        return( left);
        else return 0;
  }
  else
  {
     if ( right <=100)
		return( right);
    else return 0;
	}
  /*int vol;
  
  if ((ioctl(mix_fd, MIXER_READ(dev), &vol)) < 0)
  {
      printf("Can't read volume\n");
      return 0;
  }

  printf("Volume  = %i\n",vol);
	return(vol); */
}




int version(char *device)
{
	char dummy[256];

	fd = open(device,O_RDONLY);
	if (-1 == fd)
	{
		printf("Can't open device %s\n", device);
		return 0;/*fail*/
	}

	if (-1 != ioctl(fd,VIDIOC_QUERYCAP,dummy))
	{
		printf("Using v4l2\n");
    return 2;/*success  v4l2 */
  } 

  if (-1 != ioctl(fd,VIDIOCGCAP,dummy))
	{
		printf("Using v4l\n");
    return 1;/*success  v4l1*/
  }

  if ( (-1 == ioctl(fd,VIDIOCGCAP,dummy)) && (-1 == ioctl(fd,VIDIOC_QUERYCAP,dummy)) )
    return 0;
	
}

int initialize(int ver)
{
	if ( ver == 1)
	{
		struct video_tuner tuner;
		tuner.tuner = 0;
		if (ioctl(fd,VIDIOCGTUNER, &tuner) < 0)
			multipler = 16;
		else
		{
			if ((tuner.flags & VIDEO_TUNER_LOW) == 0)
				multipler = 16; 
			else
			multipler = 16000; 
		}
    printf("Video4Linux detected\n");
		return 1; /*success */
	} 
	
	if ( ver == 2)
	{
		struct v4l2_tuner tuner;
		tuner.input = 0;
		if (ioctl(fd,VIDIOC_G_TUNER, &tuner) < 0)
			multipler = 16;
		else
		{
			if ((tuner.capability & V4L2_TUNER_CAP_LOW) == 0)
				multipler = 16; 
			else
				multipler = 16000; 
		}
    printf("Video4Linux2 detected\n");
		return 1; /*success*/
	}
	
	if ( ver == 0)
		return 0;
	
}

void close_radio(int ver)
{
	mute(ver);
	
	if ( fd > 0)
		close(fd);
}

int setting_frequency(float freqn,int ver)
{
	int temp = ( freqn+1.0/32 )*multipler; 
	
	if ( fd < 0 ) 
		return 0; 
	
	if ( (freqn > 108) || (freqn < 65) )
		return 1; 
	
	if (ver == 2) 	
		return ioctl(fd, VIDIOC_S_FREQUENCY, &temp);
	if (ver == 1)
		return ioctl(fd, VIDIOCSFREQ, &temp);			
}

void unmute(int ver)
{
	if (ver == 2)
	{
		struct v4l2_audio vid_aud;

   	 	if (ioctl(fd, VIDIOC_G_AUDIO, &vid_aud))
			perror("VIDIOC_G_AUDIO");
	
		if ( vid_aud.mode == V4L2_AUDMODE_LOUDNESS)
			vid_aud.mode &= ~V4L2_AUDMODE_LOUDNESS; 		
			
	 	/*if (vid_aud.volume == 0)
		vid_aud.volume = 65535;
		
  	  	vid_aud.flags &= ~VIDEO_AUDIO_MUTE;*/
    
  	 	 if (ioctl(fd, VIDIOC_S_AUDIO, &vid_aud))
			perror("VIDIOC_S_AUDIO");
	}
	
	if (ver == 1)
	{
		struct video_audio vid_aud;

   	 	if (ioctl(fd, VIDIOCGAUDIO, &vid_aud))
			perror("VIDIOCGAUDIO");
		
	 	if (vid_aud.volume == 0)
			vid_aud.volume = 65535;
		
  	  	vid_aud.flags &= ~VIDEO_AUDIO_MUTE;
    
  	  	if (ioctl(fd, VIDIOCSAUDIO, &vid_aud))
			perror("VIDIOCSAUDIO");
	}
			
}

void mute(int ver)
{
	if ( ver == 2)
	{	
  	  	struct v4l2_audio vid_aud;

  	  	if (ioctl(fd, VIDIOC_G_AUDIO, &vid_aud))
			perror("VIDIOC_G_AUDIO");

  	  	vid_aud.mode |= V4L2_AUDMODE_LOUDNESS;
    
   		 if (ioctl(fd, VIDIOC_S_AUDIO, &vid_aud))
			perror("VIDIOC_S_AUDIO");
	}
	
	if ( ver == 1)
	{
		 struct video_audio vid_aud;

  	  	if (ioctl(fd, VIDIOCGAUDIO, &vid_aud))
			perror("VIDIOCGAUDIO");

  	  	vid_aud.flags |= VIDEO_AUDIO_MUTE;
    
   	 	if (ioctl(fd, VIDIOCSAUDIO, &vid_aud))
			perror("VIDIOCSAUDIO");
	}	
} 

int stereo_checking(int ver)
{
	if ( ver == 2)
	{
		struct v4l2_audio vid_aud;
	
		if ( fd < 0 )
			return -1;
		
		if ( ioctl(fd,VIDIOC_G_AUDIO,&vid_aud) < 0)
			return -1;

		if ( vid_aud.mode == V4L2_AUDMODE_STEREO_field)
			return 1; 
		else
			return 0;
	}
	
	if ( ver == 1)
	{
		struct video_audio vid_aud;
	
		if ( fd < 0 )
			return -1;
		
		if ( ioctl(fd,VIDIOCGAUDIO,&vid_aud) < 0)
			return -1;

		if ( vid_aud.mode == VIDEO_SOUND_STEREO)
			return 1; 
		else
			return 0; 
	}
	
	if ( ver == 0)
	{
		return -1;
	}		 
}

int signal_strength(int ver)
{
	if ( ver == 2)
	{
		struct v4l2_tuner vid_tun;
		int strength;
	
		if ( fd < 0)
			return -1;
	
		memset(&vid_tun,0,sizeof(vid_tun));	
		
		ioctl(fd, VIDIOC_G_TUNER, &vid_tun);
	
		strength = vid_tun.signal;
		return strength;
	}
	
	if ( ver == 1)
	{
		struct video_tuner vid_tun;
		int strength;
	
		if ( fd < 0)
			return -1;
	
		memset(&vid_tun,0,sizeof(vid_tun));	
		
		ioctl(fd, VIDIOCGTUNER, &vid_tun);
	
		strength = vid_tun.signal>>13;//now from 0 to 7
		printf("SIGNAL = %i\n",strength);
		return strength;	
	}
	
	if ( ver == 0)
	{
		return 0;	
	}	
}
        
/*NOW INFO FUNCTIONS*/
int can_stereo(int ver)
{
	if ( ver == 2)
	{
  		struct v4l2_tuner vid_tun;

  		if ( fd < 0)
			return (-1);

  		ioctl(fd, VIDIOC_G_TUNER, &vid_tun);
  		if ( vid_tun.capability & V4L2_TUNER_CAP_STEREO ) return(1);
  		else return(0); 
  	}
	
	if ( ver == 1)
	{
		  struct video_audio vid_aud;

 		 if ( fd < 0)
			return (-1);

		ioctl(fd, VIDIOCGAUDIO, &vid_aud);
   		if ( vid_aud.flags & VIDEO_SOUND_STEREO ) return(1);
   		else return(0); 
	}
	else return(-1);

	
}

int can_rds(int ver)
{
	if ( ver == 2)
	{
  		struct v4l2_tuner vid_tun;

  		if ( fd < 0)
			return (-1);

 		/*ioctl(fd, VIDIOC_G_TUNER, &vid_tun); */
 		/*if ( vid_tun.capability & VIDEO_TUNER_RDS_ON ) return("Yes");
 		else return("No");*/
 		return(-1);
	}
	
	if ( ver == 1)
	{
		  struct video_tuner vid_tun;

 		  if ( fd < 0)
			return (-1);

		ioctl(fd, VIDIOCGTUNER, &vid_tun);
   		if ( vid_tun.flags & VIDEO_TUNER_RDS_ON ) return(1);
   		else return(0);
	}
	else return(-1);
 
}


int can_mutable(int ver)
{
	if ( ver == 2)
	{
		struct v4l2_audio vid_aud;

  		if ( fd < 0)
		return (-1);

		ioctl(fd, VIDIOC_G_AUDIO, &vid_aud);
  		if ( vid_aud.capability & V4L2_AUDCAP_LOUDNESS ) return(1);
   		else return(0);
	}
	
	if ( ver == 1)
	{
	 	struct video_audio vid_aud;

  		if ( fd < 0)
		return (-1);

		ioctl(fd, VIDIOCGAUDIO, &vid_aud);
   		if ( vid_aud.flags & VIDEO_AUDIO_MUTABLE ) return(1);
   		else return(0);	
	} 
	else return(-1);

}
/*
char* give_tuner_name(int ver)
{
  char *name_radio;
  
	if ( ver == 2)
	{
   		struct v4l2_tuner vid_tun;
     
  		if ( fd < 0)
			return ("Error");

		ioctl(fd, VIDIOC_G_TUNER, &vid_tun);
  		return(vid_tun.name);
	}
	
	if ( ver == 1)
	{
		struct video_tuner vid_tun;
     
  		if ( fd < 0)
		return ("Error");

    name_radio = vid_tun.name;
    printf("TUNER : %s\n",name_radio);
		ioctl(fd, VIDIOCGTUNER, &vid_tun);
  		//return(vid_tun.name);
    return("");
	} 
	else return("Error");	
}

char* give_radio_name(int ver)
{
  char *name_radio;

  
	if ( ver == 2)
	{
  		struct v4l2_audio vid_aud;
  
  		if ( fd < 0)
			return ("Error");

		ioctl(fd, VIDIOC_G_AUDIO, &vid_aud);
  		return(vid_aud.name);
	}
	
	if ( ver == 1)
	{
	 	struct video_audio vid_aud;
  
  	 if ( fd < 0)
			return ("Error");
      
		ioctl(fd, VIDIOCGAUDIO, &vid_aud);
    		name_radio = vid_aud.name;
    		printf("VER = %i  RADIO : %s\n",ver,name_radio);
  		//return(vid_aud.name);
    	return("");
	} 
 	 else return("Error");	
}*/

